<?php 

// A quick and dirty 404 handler for a missing tiles when acessing a mapguide tiled map served directly by apache
// ie http://localhost:8008/sheboyganTiles/S13/BASE%20LAYER/R4/C4/11_7.png for a specific mapDefinition

/*

Usage:

1. Add the following to your apache httpd.conf, it exposes a map specific tile cache directory 
and adds cache headers to cache for 24 hours

--start--

<Directory "C:/Program Files/MapGuideOpenSource2.0/Server/Repositories/TileCache/Samples_Sheboygan_MapsTiled_Sheboygan/">
    Options Indexes
    ErrorDocument 404 /mapguide/seedSheboyganTile.php
	Header set Cache-Control "max-age=86400, public"
    AllowOverride None
    Order allow,deny
    Allow from all
</Directory>
Alias /sheboyganTiles/ "C:/Program Files/MapGuideOpenSource2.0/Server/Repositories/TileCache/Samples_Sheboygan_MapsTiled_Sheboygan/"

--end--

2. Save this file as C:\Program Files\MapGuideOpenSource2.0\WebServerExtensions\www\seedSheboyganTile.php

3. restart apache

*/

// ----------------   Setup ----------------------

// Location of your MapGuide web extensions installation
$extensionDir = "C:/Program Files/MapGuideOpenSource2.0/WebServerExtensions/www/";

// Tile Map that you want to serve
$tileMap = "Library://Samples/Sheboygan/MapsTiled/Sheboygan.MapDefinition";

// User and Password to use
$tileUser = 'Anonymous';
$tilePassword = '';

// ----------------   End Setup ------------------


if ( !isset($_SERVER['REDIRECT_URL'])){
  header("HTTP/1.x 500 expected SERVER REDIRECT_URL is missing");
  print "<h1>Exception</h1><p>";
  print "<strong>Error:</strong> expected SERVER REDIRECT_URL is missing<br />";
  print "<strong>Details:</strong> This is script is designed to be called via an apache ErrorDocument directive </br>";
  exit;
}

$ar_path=split("/",$_SERVER['REDIRECT_URL']); 

if ( sizeof($ar_path) != 7 ){
  header("HTTP/1.x 500 Wrong number of directories in path, " . sizeof($ar_path) . " expected 7");
  exit;
}

$basemaplayergroupname=$ar_path[3];
$scaleIndex=substr($ar_path[2],1);

$groupRow=substr($ar_path[4],1);
$groupCol=substr($ar_path[5],1);
$ar_file=split("[_.]",$ar_path[6]);
$fileRow=$ar_file[0];
$fileCol=$ar_file[1];

$tileRow=$groupRow+$fileRow;
$tileCol=$groupCol+$fileCol;

// This is stolen from MapViewerPHP/constants.php ; it's safer to include the file, but less platform-independant.

class MgServiceType
{
   /// Tile Service
   const TileService = 5 ; 
}

try 
{

  MgInitializeWebTier($extensionDir. "webconfig.ini");

  // Connect to the site as your chosen user
  $user = new MgUserInformation($tileUser,$tilePassword);
  $siteConnection = new MgSiteConnection();
  $siteConnection->Open($user);
   
  // Set up the tiling service
  $tileService = $siteConnection->CreateService(MgServiceType::TileService);

  // Fetch the image
  $tileResource = new MgResourceIdentifier($tileMap);
  $byteReader = $tileService->GetTile($tileResource, urldecode($basemaplayergroupname), $tileCol, $tileRow, $scaleIndex);

  // Output the image contents

  header("Content-Type: image/png");
  $buffer = '';
  while ($byteReader->Read($buffer, 50000) != 0)
  {
    echo $buffer;
  }

}
catch (MgException $ex) 
{
  header("HTTP/1.0 Status: 500 " . $ex->GetMessage());
}

exit;
?>
